/***************************************************************************
 *   Copyright (C) 2007 by Julien Keller   *
 *   j.keller@imperial.ac.uk   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/



#ifndef __P2BLOW3PTSFINAL_H__ //inclusion guard
#define __P2BLOW3PTSFINAL_H__

#include "m_matrix2.h"  //some procedures for standard operations on matrices
#include "plotter.h"    //some procedures for handling plotters 
#include "in-output.h"  //takes care of input and output
#include <MMatrix.h>    //library to handle matrices in memory quickly
#include <sys/time.h> //get the time to check speed

//Constants for Tweaking the Program
#define HORIZ_Z_INTEGR_OVER 2.2 //z1 is integrated from 0 to this value (subject to the cut off function) to obtain a part of the integral
#define Z2LOOP_CAP_FACTOR 1 //the cap to prevent the z2 integration loop from going on forever if it doesn't reach the cutoff is this number of times larger than the number of steps in the z1 loop


//Mathematical Constants and Macros
#define Complex  std::complex<double>
#define _Complex_I std::complex<double>(0, 1)
#define _1  (std::complex<double>(1, 0))
#define _2  (std::complex<double>(2, 0))
#define _0  (std::complex<double>(0, 0))
#define PI  3.14159265358979323846

#define pow2(x) (x*x)
#define pow3(x) (x*x*x)
#define pow4(x) pow2(x*x)
#define pow5(x) (x*pow4(x))
#define pow6(x) pow2(pow3(x))
#define pow8(x) pow2(pow4(x))


MMatrix<double> * doAlgorithm(const MMatrix<double> * p_a, MMatrix<double> * p_an, MMatrix<double>*, ResultsStore & additResults, int integrSteps, IntegrationMethod integrMethod = balanced); //carries out the main algorithm to improve the matrix of coefficients on the matrix pointed to by p_a and puts the result in the matrix pointed to by p_an, returning p_an - additResults recieves various additional useful values calculated, integrSteps controls the quality of the numerical integration, itegrMethod controls the method used.
MMatrix<double> findVolumeFormMatrix(const MMatrix<double> & a, double z1, double z2); //returns a 3x3 matrix whose determinant is the volume form (actually determinant divided by cube of element (0,0) is volume form) (more description needed)
void addIntegrationStep(MMatrix<double> & ip, const double z1, const double z2, const double factor); //perform the addition for the step in the integration for (z1,z2) - add factor * z1^(2p+1) * z2^(2q+1) to every coefficient (p,q) stored in ip
void normaliseResults(const MMatrix<double> * p_an, MMatrix<double> * p_anNorm); //normalises the matrix pointed to by p_an by scaling every element by a factor determined by the sum of the elements and places results in matrix pointed to by p_anNorm
void normaliseResults2(const MMatrix<double> * p_anNorm, MMatrix<double> * p_anNorm2); //further normalises the matrix pointed to by p_anNorm giving (1/k)(log(anNorm)), placing result in matrix pointed to by p_anNorm2 
bool inPolygon(int p, int q, int k); //returns whether the point (p,q) is in the hexagon side k, made by chopping the top left and bottom right corners off a square side 2k
double cutOffFunct(double y); //returns a number 0 to 1 with a smooth cut off in value 
double ellipseFunct(double x1, double x2); //use of the symmetries
double rotationFunct(double x1, double x2); //use of the symmetries
void compute_curvatures_local(MMatrix<double> * ,double,double,double&);
   //mathematical procedure to compute curvatures terms.
    
//////////////////////
//plotting functions//
//////////////////////
void plotDisp(OutputsStruct & outputs, const MMatrix<double> * p_an, const MMatrix<double> * p_anNorm, const MMatrix<double> * p_anNorm2, const ResultsStore & additResults, int iterationsComplete);
void printText(OutputsStruct & outputs, const MMatrix<double> * p_an, const MMatrix<double> * p_anNorm, const MMatrix<double> * p_anNorm2, const ResultsStore & additResults, int iterationsComplete);
void drawHexigon();
void plotHexMatrix(const MMatrix<double> & M, ColourGradient gradient); //plots an even grid of coloured points across a 1x1 square with one point for each element of M. Points outside the hexagon are excluded. Colours are in the colour gradient specified and indicate the relative values of the elements. M must be square and at least 2x2.
void plotHexPoints(const vector<double> & x1, const vector<double> & x2, const vector<double> & v, ColourGradient gradient); //plot the points (x1,x2) for all elements of the vector in a colour related to the equivalent element of v. Colours are in the colour gradient specified and indicate the relative values of v. x1,x2,v must be the same length.
double findHexMatrixMax(const MMatrix<double> & M); //find the maximum value in a square matrix excluding elements outside the hexagon
double findHexMatrixMin(const MMatrix<double> & M); //find the minimum value in a square matrix excluding elements outside the hexagon
double findHexMatrixMean(const MMatrix<double> & M); //find the mean of the elements of a square matrix excluding elements outside the hexagon


/////////////////////
//utility functions//
/////////////////////
double vectorMax(vector<double> v); //returns value of largest element of v - v must have at least 1 element
double vectorMin(vector<double> v); //returns value of smallest element of v - v must have at least 1 element
double vectorMean(vector<double> v); //returns mean of elements of v
void skipline(ifstream &); /* skip end of line when reading file */



/* Handle any plotter - Choices by default */

int handle;
int choice_output=0; //file of X
int zoom=2;  //size of output 

//////////////////////////////////////////////
//Choices by default -- IMPORTANT CONSTANTS //
//////////////////////////////////////////////
int integrSteps = 30; //this is the number of steps along the horizontal axis in the numerical` integration by z
int numIterations = 5; //number of iterations of whole algorithm to perform
int method=0; // 0-> balanced, 1-> ricci method , 2-> recursive balanced by 1 step , 3-> recursive ricci method by 1 step /// see README FILE
int write_result=0; // 0-> no output file, 1-> write results in resX.txt
int gchar=0; //if 1 then user has to press key between each iteration 
int print_results=0; //0-> few output , 1-> all output possible Results
int sleep_time=0; //time to sleep between 2 iterations
MMatrix <double> Mresults(4,1);  // Contains some important results : [0]=number of points, [1]=average , [2]=max, [3]=min
int random_init=0;

#endif //__P2BLOW3PTSFINAL_ - end inclusion guard

